/**
 * GUIEnvironment is the class that shows the Lunar Lander User Interface
 * @author Rosalva Gallardo
 * @version 1.0.0
 */

import java.awt.*;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Observer;

import javax.swing.*;

 
public class GUIEnvironment extends JPanel implements KeyListener, ISubject {
	
    BufferedImage backbuffer;
    int wRec=70;
    int hRec=70;

    private int keyPressed;
    boolean hit=false;
    boolean landed=false;
    SpaceCraft spaceCraft;
    Clock clock;
    private ArrayList observers;
    ObserverLL sensor;
    
	/** 
     * Lunar Lander game. Your mission is to land the space craft on the platform with a speed less than 20 
     * @param args this main method does not need arguments
     */    
    public static void main(String args[]) {
    	GUIEnvironment test = new GUIEnvironment();
    	test.init();
        JFrame app = new JFrame();
        app.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
        app.getContentPane().add (test);
        app.setSize (800,800);
        app.setBackground (Color.WHITE);
        app.setLocation (0,0);
        app.addKeyListener (test);
        app.setVisible (true);
        while (true) {
        }
    }

	/** 
     * Initialize variables and classes needed to start the game
     */   
    public void init()
    {
        hit=false;
        landed=false;
        spaceCraft = new SpaceCraft();
        observers = new ArrayList();
        clock = new Clock(700);
        sensor = new ObserverLL(this, clock, spaceCraft);    	
    	
    	
    }
    
	/** 
     * Display the User Interface of the game
     */   
    public void displayGUI() {
        int appWidth = getWidth();
        int appHeight = getHeight();
        backbuffer = (BufferedImage) createImage (appWidth, appHeight);
        Graphics2D g2 = backbuffer.createGraphics();
        g2.setRenderingHint (RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        g2.setPaint (UIManager.getColor ("Panel.background"));
        g2.setColor(Color.blue);
        g2.setFont(new Font("Courier",Font.BOLD,12));
        
        // Begin section to be refactored
        g2.drawString("Instructions", 400, 50);
        g2.drawString("Land in the orange platform with a velocity less than 20", 400, 70);
        g2.drawString("Right Arrow: Rotate right", 400, 90);
        g2.drawString("Left Arrow: Rotate left", 400, 110);
        g2.drawString("Space: Thrust", 400, 130); 
        // End section to be refactored
        
        g2.drawString("Environment Values", 500, 150);
        g2.drawString("Position X="+spaceCraft.getPositionX(), 500, 170);
        g2.drawString("Position Y="+spaceCraft.getPositionY(), 500, 190);
        g2.drawString("Velocity  ="+Math.round(spaceCraft.getVelocity()*100.0) / 100.0, 500, 210);
        g2.drawString("Fuel      ="+spaceCraft.getFuel(), 500, 230);
        g2.setColor(Color.blue);
        Rectangle rectangleGround = new Rectangle(0,750, 800, 750);
        g2.fill(rectangleGround);
        g2.draw(rectangleGround);
        g2.setColor(Color.white);
        Rectangle rectangleLunarLander = new Rectangle(0+spaceCraft.getPositionX(),0+spaceCraft.getPositionY(), wRec, hRec);
        g2.draw(rectangleLunarLander);
        g2.setColor(Color.orange); 
        Rectangle rectanglePlatform = new Rectangle(50,740,150,740);
        g2.fill(rectanglePlatform);
        g2.draw(rectanglePlatform);
        g2.setColor(Color.white);
        g2.drawString("Inf111", 500, 760);
        g2.rotate(spaceCraft.getRotation()*(Math.PI/180.0));
        g2.drawImage (spaceCraft.getImage(), spaceCraft.getPositionX(), spaceCraft.getPositionY(), this);

        if (g2.hit(rectangleLunarLander, rectanglePlatform, true) && spaceCraft.getVelocity()<20)
        {	this.landed=true;
        	g2.setColor(Color.RED);
        	g2.rotate(-1*spaceCraft.getRotation()*(Math.PI/180.0));
        	g2.drawString("Congratulations!!!!", 400, 400);
        	g2.drawString("Press a key to continue", 400,450);
        	clock.stop();
        	spaceCraft = null;
            clock = null;
            observers = null;
            sensor = null; 
        	//sensor.stop();      	
        }
        else if (g2.hit(rectangleGround, rectangleLunarLander, true))
        {	this.setHit(true);
        	g2.setColor(Color.RED);
        	g2.rotate(-1*spaceCraft.getRotation()*(Math.PI/180.0));
        	g2.drawString("Crash!!!!", 400, 400);
        	g2.drawString("Press a key to continue", 400,450);
        	//System.out.println("HIT!!!!!!!!!!!!");   
        	clock.stop();
        	spaceCraft = null;
            clock = null;
            observers = null;
            sensor = null; 
        	
        	
        }
        g2.dispose();    
        repaint();
    }

	/** 
     * Show the contents of graphics component argument
     * @param gr graphics object that contains the information to be displayed in the GUI
     * 
     */     
    protected void paintComponent (Graphics gr) {
        super.paintComponent (gr);
        if (backbuffer == null) {
        	displayGUI();
        }
        gr.drawImage (backbuffer, 0, 0, this);
    }

	/** 
     * Paint the contents of graphics argument
     * @param graph graphics object that contains the information to be displayed in the GUI
     * 
     */     
    public void paint (Graphics graph) {
    	super.paint (graph);
    }
    
	/** 
     * Update the contents of graphics argument
     * @param graph2 graphics object that contains the information to be updated
     */     
    public void update (Graphics graph2) {}

    /** 
     * Capture the event when a key is pressed
     * @param ke KeyEvent object that contains information about the key pressed
     * 
     */       
   public void keyPressed (KeyEvent ke) {
        int intKey = ke.getKeyCode();
        this.setKeyPressed(intKey);
        //System.out.println("Key pressed1");
        if (this.getHit() || this.landed)
        {
        	//System.out.println("Key pressed2");
        	this.init();
        }
        else
        {
        	notifyObservers();
        }
	    
   }
   /** 
    * Capture the event when a key is released
    * @param ke KeyEvent object that contains information about the key released
    * 
    */   
   public void keyReleased (KeyEvent ke) {}
   
   /** 
    * Capture the event when a key is typed
    * @param ke KeyEvent object that contains information about the key typed
    * 
    */    
   public void keyTyped (KeyEvent ke) {}

	/** 
    * Add an observer 
    * @param o observer to be added
    */
   public void addObserver(Observer o) {
	observers.add( o );
	// TODO Auto-generated method stub
	
   }
	/** 
    * Remove an observer 
    * @param o observer to be removed
    */
   public void removeObserver(Observer o) {
	// TODO Auto-generated method stub
	observers.remove( o );
   }	
	/** 
    * Notify all observers 
    */    
   private void notifyObservers() {
    // loop through and notify each observer
    Iterator i = observers.iterator();
    while( i.hasNext() ) {
          Observer o = ( Observer ) i.next();
          o.update( null, this );
    }
   }
	/** 
    * Set the property keyPressed 
    * @param keyPressed integer that represents the key pressed
    */   
   public void setKeyPressed(int keyPressed)
   {
	this.keyPressed = keyPressed;
   }
	/** 
    * Get the property keyPressed 
    */  
   public int getKeyPressed()
   {
	return keyPressed;
   }   
	/** 
    * Set the property hit
    * @param hit boolean that indicates if the space craft hit
    */    
   public void setHit(boolean hit)
   {
	this.hit = hit;
   }
	/** 
    * Get the property hit
    */  
   public boolean getHit()
   {
	return hit;
   }   
}